@@ -270,8 +270,19 @@ def _configure_logging(args: argparse.Namespace):
 
 
 def is_rasa_project_setup(args: argparse.Namespace, project_path: Text) -> bool:
+    """Checks if `project_path` contains a valid Rasa Open Source project.
+
+    Args:
+        args: Command-line arguments.
+        project_path: Path to the possible Rasa Open Source project.
+
+    Returns:
+        `True` if `project_path` is a valid Rasa Open Source project, `False` otherwise.
+    """
     config_path = _get_config_path(args)
-    mandatory_files = [config_path, DEFAULT_DOMAIN_PATH]
+    domain_path = _get_domain_path(args)
+
+    mandatory_files = [config_path, domain_path]
 
     for f in mandatory_files:
         if not os.path.exists(os.path.join(project_path, f)):
@@ -307,7 +318,8 @@ def _validate_rasa_x_start(args: argparse.Namespace, project_path: Text):
             "https://rasa.com/docs/rasa/command-line-interface#rasa-init)."
         )
 
-    _validate_domain(os.path.join(project_path, DEFAULT_DOMAIN_PATH))
+    domain_path = _get_domain_path(args)
+    _validate_domain(os.path.join(project_path, domain_path))
 
     if args.data and not os.path.exists(args.data):
         rasa.shared.utils.cli.print_warning(
@@ -405,6 +417,14 @@ def _get_config_path(args: argparse.Namespace,) -> Optional[Text]:
     return config_path
 
 
+def _get_domain_path(args: argparse.Namespace,) -> Optional[Text]:
+    domain_path = rasa.cli.utils.get_validated_path(
+        args.domain, "domain", DEFAULT_DOMAIN_PATH
+    )
+
+    return domain_path
+
+
 def _get_credentials_and_endpoints_paths(
     args: argparse.Namespace,
 ) -> Tuple[Optional[Text], Optional[Text]]:
@@ -464,13 +484,19 @@ def run_locally(args: argparse.Namespace) -> None:
     process = start_rasa_for_local_rasa_x(args, rasa_x_token=rasa_x_token)
 
     config_path = _get_config_path(args)
+    domain_path = _get_domain_path(args)
 
     telemetry.track_rasa_x_local()
 
     # noinspection PyBroadException
     try:
         local.main(
-            args, project_path, args.data, token=rasa_x_token, config_path=config_path
+            args,
+            project_path,
+            args.data,
+            token=rasa_x_token,
+            config_path=config_path,
+            domain_path=domain_path,
         )
     except RasaXTermsError:
         # User didn't accept the Rasa X terms.
