@@ -21,6 +21,8 @@ from typing import (
 )
 
 from boto3.dynamodb.conditions import Key
+from botocore.exceptions import ClientError
+
 import rasa.core.utils as core_utils
 import rasa.shared.utils.cli
 import rasa.shared.utils.common
@@ -346,18 +348,30 @@ class RedisTrackerStore(TrackerStore):
             timeout = self.record_exp
 
         serialised_tracker = self.serialise_tracker(tracker)
-        self.red.set(self.prefix + tracker.sender_id, serialised_tracker, ex=timeout)
+        self.red.set(
+            self.key_prefix + tracker.sender_id, serialised_tracker, ex=timeout
+        )
 
     def retrieve(self, sender_id: Text) -> Optional[DialogueStateTracker]:
-        stored = self.red.get(self.prefix + sender_id)
+        """Retrieves tracker for the latest conversation session.
+
+        The Redis key is formed by appending a prefix to sender_id.
+
+        Args:
+            sender_id: Conversation ID to fetch the tracker for.
+
+        Returns:
+            Tracker containing events from the latest conversation sessions.
+        """
+        stored = self.red.get(self.key_prefix + sender_id)
         if stored is not None:
             return self.deserialise_tracker(sender_id, stored)
         else:
             return None
 
     def keys(self) -> Iterable[Text]:
-        """Returns keys of the Redis Tracker Store"""
-        return self.red.keys(self.prefix + "*")
+        """Returns keys of the Redis Tracker Store."""
+        return self.red.keys(self.key_prefix + "*")
 
 
 class DynamoTrackerStore(TrackerStore):
@@ -402,35 +416,69 @@ class DynamoTrackerStore(TrackerStore):
         except self.client.exceptions.ResourceNotFoundException:
             table = dynamo.create_table(
                 TableName=self.table_name,
-                KeySchema=[
-                    {"AttributeName": "sender_id", "KeyType": "HASH"},
-                    {"AttributeName": "session_date", "KeyType": "RANGE"},
-                ],
+                KeySchema=[{"AttributeName": "sender_id", "KeyType": "HASH"},],
                 AttributeDefinitions=[
                     {"AttributeName": "sender_id", "AttributeType": "S"},
-                    {"AttributeName": "session_date", "AttributeType": "N"},
                 ],
                 ProvisionedThroughput={"ReadCapacityUnits": 5, "WriteCapacityUnits": 5},
             )
 
             # Wait until the table exists.
             table.meta.client.get_waiter("table_exists").wait(TableName=table_name)
-        return dynamo.Table(table_name)
+        else:
+            table = dynamo.Table(table_name)
+
+            column_names = [
+                attribute["AttributeName"] for attribute in table.attribute_definitions
+            ]
+            if "session_date" in column_names:
+                rasa.shared.utils.io.raise_warning(
+                    "Attribute 'session_date' is no longer required when using a "
+                    "DynamoDB TrackerStore. Please remove this attribute from "
+                    "any existing tables.",
+                    FutureWarning,
+                )
+
+        return table
 
     def save(self, tracker):
         """Saves the current conversation state"""
         if self.event_broker:
             self.stream_events(tracker)
-        self.db.put_item(Item=self.serialise_tracker(tracker))
+        serialized = self.serialise_tracker(tracker)
+
+        try:
+            self.db.put_item(Item=serialized)
+        except ClientError as e:
+            if "Missing the key session_date" in repr(e):
+                # the session_date attribute got removed as it was useless
+                # old databases will still contain an attribute for it though
+                # which we need to set (otherwise we are getting the error we
+                # just ran into) this section should be removed in 3.0
+                legacy_date = self._retrieve_latest_session_date(tracker.sender_id)
+
+                serialized["session_date"] = legacy_date
+                self.db.put_item(Item=serialized)
+            else:
+                raise
+
+    def _retrieve_latest_session_date(self, sender_id: Text) -> Optional[int]:
+        dialogues = self.db.query(
+            KeyConditionExpression=Key("sender_id").eq(sender_id),
+            Limit=1,
+            ScanIndexForward=False,
+        )["Items"]
+
+        if not dialogues:
+            return int(datetime.now(tz=timezone.utc).timestamp())
+
+        return dialogues[0].get("session_date")
 
     def serialise_tracker(self, tracker: "DialogueStateTracker") -> Dict:
         """Serializes the tracker, returns object with decimal types"""
         d = tracker.as_dialogue().as_dict()
         d.update(
-            {
-                "sender_id": tracker.sender_id,
-                "session_date": int(datetime.now(tz=timezone.utc).timestamp()),
-            }
+            {"sender_id": tracker.sender_id,}
         )
         # DynamoDB cannot store `float`s, so we'll convert them to `Decimal`s
         return core_utils.replace_floats_with_decimals(d)
@@ -869,33 +917,44 @@ class SQLTrackerStore(TrackerStore):
         )
 
     def _create_database_and_update_engine(self, db: Text, engine_url: "URL"):
-        """Create databse `db` and update engine to reflect the updated `engine_url`."""
-
+        """Creates database `db` and updates engine accordingly."""
         from sqlalchemy import create_engine
 
+        if not self.engine.dialect.name == "postgresql":
+            rasa.shared.utils.io.raise_warning(
+                "The parameter 'login_db' can only be used with a postgres database.",
+            )
+            return
+
         self._create_database(self.engine, db)
         engine_url.database = db
         self.engine = create_engine(engine_url)
 
     @staticmethod
-    def _create_database(engine: "Engine", db: Text):
+    def _create_database(engine: "Engine", database_name: Text) -> None:
         """Create database `db` on `engine` if it does not exist."""
-
         import psycopg2
 
         conn = engine.connect()
 
-        cursor = conn.connection.cursor()
-        cursor.execute("COMMIT")
-        cursor.execute(f"SELECT 1 FROM pg_catalog.pg_database WHERE datname = '{db}'")
-        exists = cursor.fetchone()
-        if not exists:
+        matching_rows = (
+            conn.execution_options(isolation_level="AUTOCOMMIT")
+            .execute(
+                sa.text(
+                    "SELECT 1 FROM pg_catalog.pg_database "
+                    "WHERE datname = :database_name"
+                ),
+                database_name=database_name,
+            )
+            .rowcount
+        )
+
+        if not matching_rows:
             try:
-                cursor.execute(f"CREATE DATABASE {db}")
+                conn.execute(f"CREATE DATABASE {database_name}")
             except psycopg2.IntegrityError as e:
-                logger.error(f"Could not create database '{db}': {e}")
+                logger.error(f"Could not create database '{database_name}': {e}")
 
-        cursor.close()
         conn.close()
 
     @contextlib.contextmanager
