@@ -52,6 +52,7 @@ KEY_RESPONSES = "responses"
 KEY_ACTIONS = "actions"
 KEY_FORMS = "forms"
 KEY_E2E_ACTIONS = "e2e_actions"
+KEY_RESPONSES_TEXT = "text"
 
 ALL_DOMAIN_KEYS = [
     KEY_SLOTS,
@@ -286,8 +287,8 @@ class Domain:
         slots = []
         # make a copy to not alter the input dictionary
         slot_dict = copy.deepcopy(slot_dict)
-        # Sort the slots so that the order of the slot states is consistent
-        for slot_name in sorted(slot_dict):
+        # Don't sort the slots, see https://github.com/RasaHQ/rasa-x/issues/3900
+        for slot_name in slot_dict:
             slot_type = slot_dict[slot_name].pop("type", None)
             slot_class = Slot.resolve_by_type(slot_type)
 
@@ -504,10 +505,25 @@ class Domain:
         store_entities_as_slots: bool = True,
         session_config: SessionConfig = SessionConfig.default(),
     ) -> None:
+        """Creates a `Domain`.
+
+        Args:
+            intents: Intent labels.
+            entities: The names of entities which might be present in user messages.
+            slots: Slots to store information during the conversation.
+            templates: Bot responses. If an action with the same name is executed, it
+                will send the matching response to the user.
+            action_names: Names of custom actions.
+            forms: Form names and their slot mappings.
+            action_texts: End-to-End bot utterances from end-to-end stories.
+            store_entities_as_slots: If `True` Rasa will automatically create `SlotSet`
+                events for entities if there are slots with the same name as the entity.
+            session_config: Configuration for conversation sessions. Conversations are
+                restarted at the end of a session.
+        """
         self.entities, self.roles, self.groups = self.collect_entity_properties(
             entities
         )
-
         self.intent_properties = self.collect_intent_properties(
             intents, self.entities, self.roles, self.groups
         )
@@ -520,7 +536,6 @@ class Domain:
         )
         action_names += overridden_form_actions
 
-        self.slots = slots
         self.templates = templates
         self.action_texts = action_texts or []
         self.session_config = session_config
@@ -530,8 +545,9 @@ class Domain:
         # only includes custom actions and utterance actions
         self.user_actions = self._combine_with_templates(action_names, templates)
 
-        # includes all actions (custom, utterance, default actions and forms)
-        self.action_names = (
+        # includes all action names (custom, utterance, default actions and forms)
+        # and action texts from end-to-end bot utterances
+        self.action_names_or_texts = (
             self._combine_user_with_default_actions(self.user_actions)
             + [
                 form_name
@@ -541,6 +557,10 @@ class Domain:
             + self.action_texts
         )
 
+        self._user_slots = copy.copy(slots)
+        self.slots = slots
+        self._add_default_slots()
+
         self.store_entities_as_slots = store_entities_as_slots
         self._check_domain_sanity()
 
@@ -641,7 +661,7 @@ class Domain:
         ] = rasa.shared.utils.common.sort_list_of_dicts_by_first_key(
             self_as_dict[KEY_INTENTS]
         )
-        self_as_dict[KEY_ACTIONS] = self.action_names
+        self_as_dict[KEY_ACTIONS] = self.action_names_or_texts
         return rasa.shared.utils.io.get_dictionary_fingerprint(self_as_dict)
 
     @rasa.shared.utils.common.lazy_property
@@ -651,11 +671,25 @@ class Domain:
         return self.user_actions + self.form_names
 
     @rasa.shared.utils.common.lazy_property
-    def num_actions(self):
-        """Returns the number of available actions."""
+    def action_names(self) -> List[Text]:
+        """Returns action names or texts."""
+        # Raise `DeprecationWarning` instead of `FutureWarning` as we only want to
+        # notify developers about the deprecation (e.g. developers who are using the
+        # Python API or writing custom policies). End users can't change anything
+        # about this warning except making their developers change any custom code
+        # which calls this.
+        rasa.shared.utils.io.raise_warning(
+            f"{Domain.__name__}.{Domain.action_names.__name__} "
+            f"is deprecated and will be removed version 3.0.0.",
+            category=DeprecationWarning,
+        )
+        return self.action_names_or_texts
 
+    @rasa.shared.utils.common.lazy_property
+    def num_actions(self) -> int:
+        """Returns the number of available actions."""
         # noinspection PyTypeChecker
-        return len(self.action_names)
+        return len(self.action_names_or_texts)
 
     @rasa.shared.utils.common.lazy_property
     def num_states(self):
@@ -683,7 +717,13 @@ class Domain:
         """
         return rasa.shared.nlu.constants.RESPONSE_IDENTIFIER_DELIMITER in template[0]
 
-    def add_categorical_slot_default_value(self) -> None:
+    def _add_default_slots(self) -> None:
+        """Sets up the default slots and slot values for the domain."""
+        self._add_requested_slot()
+        self._add_knowledge_base_slots()
+        self._add_categorical_slot_default_value()
+
+    def _add_categorical_slot_default_value(self) -> None:
         """Add a default value to all categorical slots.
 
         All unseen values found for the slot will be mapped to this default value
@@ -692,7 +732,17 @@ class Domain:
         for slot in [s for s in self.slots if isinstance(s, CategoricalSlot)]:
             slot.add_default_value()
 
-    def add_requested_slot(self) -> None:
+    def add_categorical_slot_default_value(self) -> None:
+        """See `_add_categorical_slot_default_value` for docstring."""
+        rasa.shared.utils.io.raise_deprecation_warning(
+            f"'{self.add_categorical_slot_default_value.__name__}' is deprecated and "
+            f"will be removed in Rasa Open Source 3.0.0. This method is now "
+            f"automatically called when the Domain is created which makes a manual "
+            f"call superfluous."
+        )
+        self._add_categorical_slot_default_value()
+
+    def _add_requested_slot(self) -> None:
         """Add a slot called `requested_slot` to the list of slots.
 
         The value of this slot will hold the name of the slot which the user
@@ -708,10 +758,20 @@ class Domain:
                 )
             )
 
-    def add_knowledge_base_slots(self) -> None:
-        """
-        Add slots for the knowledge base action to the list of slots, if the
-        default knowledge base action name is present.
+    def add_requested_slot(self) -> None:
+        """See `_add_categorical_slot_default_value` for docstring."""
+        rasa.shared.utils.io.raise_deprecation_warning(
+            f"'{self.add_requested_slot.__name__}' is deprecated and "
+            f"will be removed in Rasa Open Source 3.0.0. This method is now "
+            f"automatically called when the Domain is created which makes a manual "
+            f"call superfluous."
+        )
+        self._add_requested_slot()
+
+    def _add_knowledge_base_slots(self) -> None:
+        """Add slots for the knowledge base action to slots.
+
+        Slots are only added if the default knowledge base action name is present.
 
         As soon as the knowledge base action is not experimental anymore, we should
         consider creating a new section in the domain file dedicated to knowledge
@@ -719,7 +779,7 @@ class Domain:
         """
         if (
             rasa.shared.core.constants.DEFAULT_KNOWLEDGE_BASE_ACTION
-            in self.action_names
+            in self.action_names_or_texts
         ):
             logger.warning(
                 "You are using an experiential feature: Action '{}'!".format(
@@ -736,26 +796,63 @@ class Domain:
                 if s not in slot_names:
                     self.slots.append(TextSlot(s, influence_conversation=False))
 
-    def index_for_action(self, action_name: Text) -> Optional[int]:
-        """Look up which action index corresponds to this action name."""
+    def add_knowledge_base_slots(self) -> None:
+        """See `_add_categorical_slot_default_value` for docstring."""
+        rasa.shared.utils.io.raise_deprecation_warning(
+            f"'{self.add_knowledge_base_slots.__name__}' is deprecated and "
+            f"will be removed in Rasa Open Source 3.0.0. This method is now "
+            f"automatically called when the Domain is created which makes a manual "
+            f"call superfluous."
+        )
+        self._add_knowledge_base_slots()
 
+    def index_for_action(self, action_name: Text) -> Optional[int]:
+        """Looks up which action index corresponds to this action name."""
         try:
-            return self.action_names.index(action_name)
+            return self.action_names_or_texts.index(action_name)
         except ValueError:
             self.raise_action_not_found_exception(action_name)
 
-    def raise_action_not_found_exception(self, action_name) -> NoReturn:
-        action_names = "\n".join([f"\t - {a}" for a in self.action_names])
+    def raise_action_not_found_exception(self, action_name_or_text: Text) -> NoReturn:
+        """Raises exception if action name or text not part of the domain or stories.
+
+        Args:
+            action_name_or_text: Name of an action or its text in case it's an
+                end-to-end bot utterance.
+
+        Raises:
+            ActionNotFoundException: If `action_name_or_text` are not part of this
+                domain.
+        """
+        action_names = "\n".join([f"\t - {a}" for a in self.action_names_or_texts])
         raise ActionNotFoundException(
-            f"Cannot access action '{action_name}', "
+            f"Cannot access action '{action_name_or_text}', "
             f"as that name is not a registered "
             f"action for this domain. "
             f"Available actions are: \n{action_names}"
         )
 
     def random_template_for(self, utter_action: Text) -> Optional[Dict[Text, Any]]:
+        """Returns a random response for an action name.
+
+        Args:
+            utter_action: The name of the utter action.
+
+        Returns:
+            A response for an utter action.
+        """
         import numpy as np
 
+        # Raise `DeprecationWarning` instead of `FutureWarning` as we only want to
+        # notify developers about the deprecation (e.g. developers who are using the
+        # Python API or writing custom policies). End users can't change anything
+        # about this warning except making their developers change any custom code
+        # which calls this.
+        rasa.shared.utils.io.raise_warning(
+            f"'{Domain.__name__}.{Domain.random_template_for.__class__}' "
+            f"is deprecated and will be removed version 3.0.0.",
+            category=DeprecationWarning,
+        )
         if utter_action in self.templates:
             return np.random.choice(self.templates[utter_action])
         else:
@@ -822,12 +919,11 @@ class Domain:
     @rasa.shared.utils.common.lazy_property
     def input_states(self) -> List[Text]:
         """Returns all available states."""
-
         return (
             self.intents
             + self.entity_states
             + self.slot_states
-            + self.action_names
+            + self.action_names_or_texts
             + self.form_names
         )
 
@@ -884,10 +980,13 @@ class Domain:
 
         # filter entities based on intent config
         # sub_state will be transformed to frozenset therefore we need to
-        # convert the list to the tuple
+        # convert the set to the tuple
         # sub_state is transformed to frozenset because we will later hash it
         # for deduplication
-        entities = tuple(self._get_featurized_entities(latest_message))
+        entities = tuple(
+            self._get_featurized_entities(latest_message)
+            & set(sub_state.get(rasa.shared.nlu.constants.ENTITIES, ()))
+        )
         if entities:
             sub_state[rasa.shared.nlu.constants.ENTITIES] = entities
         else:
@@ -1033,9 +1132,12 @@ class Domain:
             return True
 
     def _slot_definitions(self) -> Dict[Any, Dict[str, Any]]:
-        return {slot.name: slot.persistence_info() for slot in self.slots}
+        # Only persist slots defined by the user. We add the default slots on the
+        # fly when loading the domain.
+        return {slot.name: slot.persistence_info() for slot in self._user_slots}
 
     def as_dict(self) -> Dict[Text, Any]:
+        """Return serialized `Domain`."""
         return {
             "config": {"store_entities_as_slots": self.store_entities_as_slots},
             SESSION_CONFIG_KEY: {
@@ -1046,11 +1148,38 @@ class Domain:
             KEY_ENTITIES: self._transform_entities_for_file(),
             KEY_SLOTS: self._slot_definitions(),
             KEY_RESPONSES: self.templates,
-            KEY_ACTIONS: self._custom_actions,  # class names of the actions
+            KEY_ACTIONS: self._custom_actions,
             KEY_FORMS: self.forms,
             KEY_E2E_ACTIONS: self.action_texts,
         }
 
+    @staticmethod
+    def get_responses_with_multilines(
+        responses: Dict[Text, List[Dict[Text, Any]]]
+    ) -> Dict[Text, List[Dict[Text, Any]]]:
+        """Returns `responses` with preserved multilines in the `text` key.
+
+        Args:
+            responses: Original `responses`.
+
+        Returns:
+            `responses` with preserved multilines in the `text` key.
+        """
+        from ruamel.yaml.scalarstring import LiteralScalarString
+
+        final_responses = responses.copy()
+        for response_name, examples in final_responses.items():
+            for i, example in enumerate(examples):
+                response_text = example.get(KEY_RESPONSES_TEXT, "")
+                if not response_text or "\n" not in response_text:
+                    continue
+                # Has new lines, use `LiteralScalarString`
+                final_responses[response_name][i][
+                    KEY_RESPONSES_TEXT
+                ] = LiteralScalarString(response_text)
+
+        return final_responses
+
     def _transform_intents_for_file(self) -> List[Union[Text, Dict[Text, Any]]]:
         """Transform intent properties for displaying or writing into a domain file.
 
@@ -1194,6 +1323,10 @@ class Domain:
             domain_data.update(self.cleaned_domain())
         else:
             domain_data.update(self.as_dict())
+        if domain_data.get(KEY_RESPONSES, {}):
+            domain_data[KEY_RESPONSES] = self.get_responses_with_multilines(
+                domain_data[KEY_RESPONSES]
+            )
 
         return rasa.shared.utils.io.dump_obj_as_yaml_to_string(
             domain_data, should_preserve_key_order=True
@@ -1214,7 +1347,7 @@ class Domain:
         Excludes slots which aren't featurized.
         """
 
-        return [s.name for s in self.slots if s.influence_conversation]
+        return [s.name for s in self._user_slots if s.influence_conversation]
 
     @property
     def _actions_for_domain_warnings(self) -> List[Text]:
@@ -1326,13 +1459,12 @@ class Domain:
         def check_mappings(
             intent_properties: Dict[Text, Dict[Text, Union[bool, List]]]
         ) -> List[Tuple[Text, Text]]:
-            """Check whether intent-action mappings use proper action names."""
-
+            """Checks whether intent-action mappings use valid action names or texts."""
             incorrect = []
             for intent, properties in intent_properties.items():
                 if "triggers" in properties:
                     triggered_action = properties.get("triggers")
-                    if triggered_action not in self.action_names:
+                    if triggered_action not in self.action_names_or_texts:
                         incorrect.append((intent, str(triggered_action)))
             return incorrect
 
@@ -1381,7 +1513,7 @@ class Domain:
                     )
             return message
 
-        duplicate_actions = get_duplicates(self.action_names)
+        duplicate_actions = get_duplicates(self.action_names_or_texts)
         duplicate_slots = get_duplicates([s.name for s in self.slots])
         duplicate_entities = get_duplicates(self.entities)
         incorrect_mappings = check_mappings(self.intent_properties)
@@ -1408,7 +1540,7 @@ class Domain:
 
         utterances = [
             a
-            for a in self.action_names
+            for a in self.action_names_or_texts
             if a.startswith(rasa.shared.constants.UTTER_PREFIX)
         ]
 
